defmodule Beanstream.ProfileCard do
  @moduledoc """
  Manage cards on customer profiles.
  """
  alias Beanstream.Gateway

  @spec list(Gateway.t, String.t) :: tuple
  def list(gw, customer_code) do
    Beanstream.get("profiles/#{customer_code}/cards", auth_header(gw))
    |> Beanstream.process_response
  end

  @spec make(Gateway.t, String.t, Card.t) :: tuple
  def make(gw, customer_code, req) do
    Beanstream.post("profiles/#{customer_code}/cards", %{"card" => req}, auth_header(gw))
    |> Beanstream.process_response
  end

  @doc """
  update customer profile card expiry (but not card number)
  """
  @spec update(Gateway.t, String.t, String.t, Card.t) :: tuple
  def update(gw, customer_code, card_id, req) do
    Beanstream.put("profiles/#{customer_code}/cards/#{card_id}", %{"card" => req}, auth_header(gw))
    |> Beanstream.process_response
  end

  @spec delete(Gateway.t, String.t, String.t) :: any
  def delete(gw, customer_code, card_id) do
    result = Beanstream.delete("profiles/#{customer_code}/cards/#{card_id}", auth_header(gw))
    |> Beanstream.process_response

    case result do
    {:ok, _} -> :ok
    _ -> result
    end
  end

  defp auth_header(gw) do
    Gateway.auth_header(gw, "profiles")
  end

end
